SystemOrganization addCategory: #MkKatas!
SystemOrganization addCategory: #'MkKatas-Tests'!

TestCase subclass: #BowlingGameTest
	instanceVariableNames: 'game'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MkKatas-Tests'!

!BowlingGameTest methodsFor: 'tests' stamp: 'MathiasKegelmann 4/19/2011 17:09'!
roll: x times: y
	x timesRepeat: [ game roll: y ]! !

!BowlingGameTest methodsFor: 'tests' stamp: 'MathiasKegelmann 4/19/2011 17:06'!
setUp
	game := BowlingGame new! !

!BowlingGameTest methodsFor: 'tests' stamp: 'MathiasKegelmann 4/19/2011 17:15'!
spare: firstRoll
	game roll: firstRoll.
	game roll: 10 - firstRoll.! !

!BowlingGameTest methodsFor: 'tests' stamp: 'MathiasKegelmann 4/19/2011 17:30'!
strike
	game roll: 10! !

!BowlingGameTest methodsFor: 'tests' stamp: 'MathiasKegelmann 4/19/2011 17:15'!
testAllGuttersCountsZero
	self roll: 20 times: 0.
	self assert: 0 equals: game score! !

!BowlingGameTest methodsFor: 'tests' stamp: 'MathiasKegelmann 4/19/2011 17:15'!
testAllOnes
	self roll: 20 times: 1.
	self assert: 20 equals: game score! !

!BowlingGameTest methodsFor: 'tests' stamp: 'MathiasKegelmann 4/19/2011 17:30'!
testAllSpares
	10 timesRepeat: [self spare: 5].
	game roll: 3.
	self assert: 15 * 9 + 10 + 3 equals: game score! !

!BowlingGameTest methodsFor: 'tests' stamp: 'MathiasKegelmann 4/20/2011 22:00'!
testExcessiveRollsRaiseException
	self roll: 13 times: 10.
	self should: [game score] raise: Error.! !

!BowlingGameTest methodsFor: 'tests' stamp: 'MathiasKegelmann 4/19/2011 18:00'!
testIncompleteGameRaisesException
	self roll: 11 times: 10.
	self should: [game score] raise: Error.! !

!BowlingGameTest methodsFor: 'tests' stamp: 'MathiasKegelmann 4/19/2011 17:28'!
testOneSpareAndAllOnes
	self spare: 5.
	self roll: 18 times: 1.
	self assert: 29 equals: game score! !

!BowlingGameTest methodsFor: 'tests' stamp: 'MathiasKegelmann 4/19/2011 17:30'!
testOneStrikeAndAllOnes
	self strike.
	self roll: 18 times: 1.
	self assert: 30 equals: game score! !

!BowlingGameTest methodsFor: 'tests' stamp: 'MathiasKegelmann 4/19/2011 17:44'!
testPerfectGame
	self roll: 12 times: 10.
	self assert: 300 equals: game score! !

TestCase subclass: #StringCalculatorTest
	instanceVariableNames: 'calculator'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MkKatas-Tests'!

!StringCalculatorTest methodsFor: 'tests' stamp: 'MathiasKegelmann 4/19/2011 17:01'!
setUp
	calculator := StringCalculator new! !

!StringCalculatorTest methodsFor: 'tests' stamp: 'MathiasKegelmann 4/19/2011 17:02'!
testEmptyStringShouldYieldZero
	self assert: (calculator evaluate: '') equals: 0.! !

!StringCalculatorTest methodsFor: 'tests' stamp: 'MathiasKegelmann 4/19/2011 17:02'!
testSeveralNumbersSeparatedByCommasShouldBeSummed
	self assert: (calculator evaluate: '4,5,1,3') equals: 13.! !

!StringCalculatorTest methodsFor: 'tests' stamp: 'MathiasKegelmann 4/19/2011 17:02'!
testShouldCopeWithSpace
	self assert: (calculator evaluate: ' 4,  5 ,1 ,3') equals: 13.! !

!StringCalculatorTest methodsFor: 'tests' stamp: 'MathiasKegelmann 4/19/2011 17:02'!
testShouldCopeWithWhitespaceOnlyString
	self assert: (calculator evaluate: '   ') equals: 0.! !

!StringCalculatorTest methodsFor: 'tests' stamp: 'MathiasKegelmann 4/19/2011 17:02'!
testSingleNumberShouldBeReturned
	self assert: (calculator evaluate: '42') equals: 42.! !

!StringCalculatorTest methodsFor: 'tests' stamp: 'MathiasKegelmann 4/19/2011 17:02'!
testTwoNumbersSeparatedByCommasShouldBeSummed
	self assert: (calculator evaluate: '4,5') equals: 9.! !

TestCase subclass: #WordWrapper2Test
	instanceVariableNames: 'cr'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MkKatas-Tests'!

!WordWrapper2Test methodsFor: 'tests' stamp: 'MathiasKegelmann 4/22/2011 20:18'!
setUp
	cr := String cr! !

!WordWrapper2Test methodsFor: 'tests' stamp: 'MathiasKegelmann 4/25/2011 14:10'!
testMocketry
	[:mock |
		[mock answer: 42.]
			should strictly satisfy: [mock answer: 42. mock ferz useArbitrarily]
	] runScenario ! !

!WordWrapper2Test methodsFor: 'tests' stamp: 'MathiasKegelmann 4/22/2011 20:35'!
testOnlyNecessaryLineBreakAtWordBoundary
	self assert: 'hello my', cr, 'world' equals: (WordWrapper2 wrap: 'hello my world' at: 10).! !

!WordWrapper2Test methodsFor: 'tests' stamp: 'MathiasKegelmann 4/22/2011 20:25'!
testSimpleLineBreakAtWordBoundary
	self assert: 'hello', cr, 'world' equals: (WordWrapper2 wrap: 'hello world' at: 10).! !

!WordWrapper2Test methodsFor: 'tests' stamp: 'MathiasKegelmann 4/22/2011 20:20'!
testTrivialCase
	self assert: '' equals: (WordWrapper2 wrap: '' at: 1).! !

!WordWrapper2Test methodsFor: 'tests' stamp: 'MathiasKegelmann 4/22/2011 22:06'!
testVeryLongWordShouldBeWrappedSeveralTimes
	self assert: 'hel' , cr , 'lo-', cr, 'wor', cr, 'ld' equals: (WordWrapper2 wrap: 'hello-world' at: 3)! !

!WordWrapper2Test methodsFor: 'tests' stamp: 'MathiasKegelmann 4/22/2011 22:06'!
testVerySmallMargin
	self assert: 'tes' , cr , 't', cr, 'tes', cr, 't' equals: (WordWrapper2 wrap: 'test test' at: 3)! !

!WordWrapper2Test methodsFor: 'tests' stamp: 'MathiasKegelmann 4/22/2011 22:06'!
testVerySmallMargin2
	self assert: 'te' , cr , 'st', cr, 'te', cr, 'st' equals: (WordWrapper2 wrap: 'test test' at: 2)! !

!WordWrapper2Test methodsFor: 'tests' stamp: 'MathiasKegelmann 4/22/2011 20:20'!
testWordShorterThanMargin
	self assert: 'hello' equals: (WordWrapper2 wrap: 'hello' at: 10).! !

!WordWrapper2Test methodsFor: 'tests' stamp: 'MathiasKegelmann 4/25/2011 14:15'!
testWrapWith
	[:wrapperMock | [
		WordWrapper2 wrap: 'hello this is a test'with: wrapperMock
		] should strictly satisfy: [
			wrapperMock add: 'hello';
				add: 'this';
				add: 'is';
				add: 'a';
				add: 'test';
				contents
		]
	] runScenario ! !

!WordWrapper2Test methodsFor: 'tests' stamp: 'MathiasKegelmann 4/22/2011 21:59'!
testWrapperBreakInWord
	self assert: 'tes', cr, 't' equals: ((WordWrapper2 withMargin: 3) add: 'test') contents 
! !

!WordWrapper2Test methodsFor: 'tests' stamp: 'MathiasKegelmann 4/22/2011 21:49'!
testWrapperOnlyNecessaryLineBreakAtWordBoundary
	|wrapper|
	wrapper := WordWrapper2 withMargin: 10.
	#('test' 'test' 'test') do: [:each | wrapper add: each].
	self assert: 'test test', cr, 'test' equals: wrapper contents ! !

!WordWrapper2Test methodsFor: 'tests' stamp: 'MathiasKegelmann 4/22/2011 21:43'!
testWrapperTrivialCase
	self assert: '' equals: (WordWrapper2 withMargin: 1) contents.! !

!WordWrapper2Test methodsFor: 'tests' stamp: 'MathiasKegelmann 4/22/2011 21:47'!
testWrapperWordShorterThanMargin
	self assert: 'hello' equals: ((WordWrapper2 withMargin: 10) add: 'hello') contents .! !

TestCase subclass: #WordWrapperTest
	instanceVariableNames: 'cr'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MkKatas-Tests'!

!WordWrapperTest methodsFor: 'tests' stamp: 'MathiasKegelmann 4/22/2011 20:19'!
setUp
	cr := String cr! !

!WordWrapperTest methodsFor: 'tests' stamp: 'MathiasKegelmann 4/20/2011 16:14'!
testBreakAfterSpace
	self assert: 'ab', cr, 'cd' equals: (WordWrapper wrap: 'ab cd' at: 3).! !

!WordWrapperTest methodsFor: 'tests' stamp: 'MathiasKegelmann 4/20/2011 16:01'!
testBreakAtSpace
	self assert: 'hello' , cr , 'world' equals: (WordWrapper wrap: 'hello world' at: 10)! !

!WordWrapperTest methodsFor: 'tests' stamp: 'MathiasKegelmann 4/20/2011 16:15'!
testBreakBeforeSpace
	self assert: 'ab', cr, 'cd' equals: (WordWrapper wrap: 'ab cd' at: 2).! !

!WordWrapperTest methodsFor: 'tests' stamp: 'MathiasKegelmann 4/20/2011 15:54'!
testLongWordShouldBeWrapped
	self assert: 'hel' , cr , 'lo' equals: (WordWrapper wrap: 'hello' at: 3)! !

!WordWrapperTest methodsFor: 'tests' stamp: 'MathiasKegelmann 4/20/2011 15:28'!
testTrivialCase
	self assert: '' equals: (WordWrapper wrap: '' at: 1).! !

!WordWrapperTest methodsFor: 'tests' stamp: 'MathiasKegelmann 4/20/2011 15:54'!
testVeryLongWordShouldBeWrappedSeveralTimes
	self assert: 'hel' , cr , 'lo-', cr, 'wor', cr, 'ld' equals: (WordWrapper wrap: 'hello-world' at: 3)! !

!WordWrapperTest methodsFor: 'tests' stamp: 'MathiasKegelmann 4/20/2011 16:36'!
testVerySmallMargin
	self assert: 'tes' , cr , 't', cr, 'tes', cr, 't' equals: (WordWrapper wrap: 'test test' at: 3)! !

!WordWrapperTest methodsFor: 'tests' stamp: 'MathiasKegelmann 4/20/2011 16:37'!
testVerySmallMargin2
	self assert: 'te' , cr , 'st', cr, 'te', cr, 'st' equals: (WordWrapper wrap: 'test test' at: 2)! !

!WordWrapperTest methodsFor: 'tests' stamp: 'MathiasKegelmann 4/20/2011 15:32'!
testWordShorterThanMargin
	self assert: 'hello' equals: (WordWrapper wrap: 'hello' at: 10).! !

Object subclass: #BowlingGame
	instanceVariableNames: 'rolls maxScoredIndex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MkKatas'!

!BowlingGame methodsFor: 'accessing' stamp: 'MathiasKegelmann 4/20/2011 22:02'!
frameScoreAt: frameIndex
	self updateMaxScoredIndex: frameIndex + 1.
	^ (rolls at: frameIndex) + (rolls at: frameIndex + 1)! !

!BowlingGame methodsFor: 'initialize-release' stamp: 'MathiasKegelmann 4/19/2011 17:18'!
initialize 
	rolls := OrderedCollection new.! !

!BowlingGame methodsFor: 'accessing' stamp: 'MathiasKegelmann 4/20/2011 21:58'!
isSpareAt: frameIndex
	^ (self frameScoreAt: frameIndex) = 10! !

!BowlingGame methodsFor: 'accessing' stamp: 'MathiasKegelmann 4/20/2011 21:58'!
isStrikeAt: frameIndex
	^ (rolls at: frameIndex)= 10! !

!BowlingGame methodsFor: 'accessing' stamp: 'MathiasKegelmann 4/20/2011 22:01'!
resetMaxScoredIndex
	maxScoredIndex := 0! !

!BowlingGame methodsFor: 'accessing' stamp: 'MathiasKegelmann 4/19/2011 17:18'!
roll: aSmallInteger 
	rolls add: aSmallInteger.
! !

!BowlingGame methodsFor: 'accessing' stamp: 'MathiasKegelmann 4/20/2011 22:04'!
score
	| total i |
	total := 0.
	i := 1.
	self resetMaxScoredIndex.
	10 timesRepeat: [
		(self isStrikeAt: i) ifTrue: [
			total := total + 10 + (self strikeBonusAt: i).
			i := i + 1]
		ifFalse: [
			total := total + (self frameScoreAt: i).
			(self isSpareAt: i) ifTrue: [total := total + (self spareBonusAt: i)].
			i := i + 2]].
	maxScoredIndex = rolls size ifFalse: [self error: 'Excessive rolls'].
	^ total! !

!BowlingGame methodsFor: 'accessing' stamp: 'MathiasKegelmann 4/20/2011 22:03'!
spareBonusAt: frameIndex
	self updateMaxScoredIndex: frameIndex + 2.
	^ rolls at: frameIndex + 2! !

!BowlingGame methodsFor: 'accessing' stamp: 'MathiasKegelmann 4/20/2011 22:03'!
strikeBonusAt: frameIndex
	self updateMaxScoredIndex: frameIndex + 2.
	^ (rolls at: frameIndex + 1) + (rolls at: frameIndex + 2)! !

!BowlingGame methodsFor: 'accessing' stamp: 'MathiasKegelmann 4/20/2011 22:02'!
updateMaxScoredIndex: i
	maxScoredIndex := maxScoredIndex max: i! !

Object subclass: #StringCalculator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MkKatas'!

!StringCalculator methodsFor: 'accessing' stamp: 'MathiasKegelmann 4/19/2011 16:59'!
evaluate: aString 
	aString withBlanksTrimmed ifEmpty: [^ 0].
	^ ((aString subStrings: ',') collect: [:s | s withBlanksTrimmed asNumber]) sum! !

Object subclass: #WordWrapper
	instanceVariableNames: 'margin'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MkKatas'!

!WordWrapper classSide methodsFor: 'accessing' stamp: 'MathiasKegelmann 4/20/2011 16:00'!
withMargin: anInteger
	| wrapper |
	wrapper := self basicNew.
	wrapper margin: anInteger.
	^ wrapper.
! !

!WordWrapper classSide methodsFor: 'accessing' stamp: 'MathiasKegelmann 4/20/2011 16:00'!
wrap: string at: margin
	^ (self withMargin: margin) wrap: string.
! !

!WordWrapper methodsFor: 'accessing' stamp: 'MathiasKegelmann 4/20/2011 16:30'!
breakLine: string at: x gap: y
	^ (string first: x) , String cr , (self wrap: (string allButFirst: x + y))! !

!WordWrapper methodsFor: 'accessing' stamp: 'MathiasKegelmann 4/20/2011 15:58'!
margin: anInteger
	margin := anInteger! !

!WordWrapper methodsFor: 'accessing' stamp: 'MathiasKegelmann 4/20/2011 21:43'!
wrap: string
	^ string size <= margin
		ifTrue: [ string ]
		ifFalse: [ 
			(string at: margin + 1) = Character space
				ifTrue: [	self breakLine: string at: margin gap: 1 ]
				ifFalse: [ 
					| space |
					space := (string first: margin) findLastOccurrenceOfString: ' ' startingAt: 1.
					space = 0
						ifTrue: [ self breakLine: string at: margin gap: 0]
						ifFalse: [ self breakLine: string at: space - 1 gap: 1]]]! !

Object subclass: #WordWrapper2
	instanceVariableNames: 'contents line margin'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MkKatas'!

!WordWrapper2 classSide methodsFor: 'instance creation' stamp: 'MathiasKegelmann 4/22/2011 21:46'!
withMargin: aSmallInteger 
	|wrapper|
	wrapper := WordWrapper2 new.
	wrapper margin: aSmallInteger.
	^wrapper! !

!WordWrapper2 classSide methodsFor: 'accessing' stamp: 'MathiasKegelmann 4/25/2011 14:16'!
wrap: aString at: margin
	^ self wrap: aString with: (WordWrapper2 withMargin: margin)
! !

!WordWrapper2 classSide methodsFor: 'private' stamp: 'MathiasKegelmann 4/25/2011 14:14'!
wrap: aString with: aWrapper
	aString subStrings do: [ :word | aWrapper add: word].
	^ aWrapper contents! !

!WordWrapper2 methodsFor: 'accessing' stamp: 'MathiasKegelmann 4/22/2011 22:05'!
add: aByteString 
	line 
		ifEmpty: [ |word|
			word := aByteString.
			[word size > margin] whileTrue: [
				contents add: (word first: margin).
				word := word allButFirst: margin
			].
			line :=word] 
		ifNotEmpty: [	
			line size + 1 + aByteString size > margin 
				ifTrue: [
					contents add: line.
					line := ''.
					self add: aByteString] 
				ifFalse: [
					line := line, ' ', aByteString]]! !

!WordWrapper2 methodsFor: 'accessing' stamp: 'MathiasKegelmann 4/22/2011 21:55'!
contents
	| result |
	result := OrderedCollection withAll: contents.
	line ifNotEmpty: [result add: line].
	^ String cr join: result! !

!WordWrapper2 methodsFor: 'accessing' stamp: 'MathiasKegelmann 4/22/2011 20:36'!
initialize 
	contents := OrderedCollection new.
	line := ''.! !

!WordWrapper2 methodsFor: 'initialization' stamp: 'MathiasKegelmann 4/22/2011 21:44'!
margin: aSmallInteger 
	margin := aSmallInteger ! !
